#include "../../proj/tl_common.h"
#include "../../proj/mcu/watchdog_i.h"
#include "../../proj_lib/rf_drv.h"
#include "../../proj_lib/pm.h"
#include "../common/rf_frame.h"
#include "../common/blt_led.h"
#include "../link_layer/rf_ll.h"
#include "../common/common_emi.h"


#include "mouse_info.h"
#include "mouse.h"
#include "mouse_rf.h"
#include "mouse_button.h"
#include "mouse_wheel.h"
#include "mouse_sensor_pix.h"
#include "mouse_sensor.h"
#include "mouse_rf.h"
#include "mouse_custom.h"
#include "trace.h"
#include "mouse_pm.h"
#include "drv_mouse_pmw3610.h"

#include "../../proj/drivers/uart.h"

mouse_status_t   mouse_status;

mouse_hw_t       mouse_hw;

mouse_data_t     mouse_data;

#if(MOUSE_GOLDEN_VPATTERN)
extern u8 signal_battery;
#endif
extern mouse_sleep_t mouse_sleep;
void mouse_pair_and_sync_process(mouse_status_t * mouse_status);

void platform_init( mouse_status_t *pStatus )
{
	//sleep_us(1000000);
#if(DAUL_MOUSE_YJX_BOARD )
	gpio_set_func(GPIO_SWS, AS_GPIO);
#endif


	mouse_hw_t *pHW = pStatus->hw_define;

	mouse_button_init(pHW);			/*********** Button Init ****************/
    mouse_wheel_init(pHW);			/*********** Wheel Init ****************/
#if(MOUSE_SENSOR_MODULE_EN)

#if(DUAL_MOUSE_BOARD != DUAL_MOUSE_MIMOUSE_BOARD)
	mouse_sensor_hw_init(pHW, &pStatus->mouse_sensor, (pStatus->mouse_mode == STATE_POWERON));
#else
	drv_mouse_sensor_init();
#endif



#endif

#if MOUSE_SW_CUS
//#define led_cnt_rate    (8 << (pStatus->high_end == MS_HIGHEND_250_REPORTRATE))
	//mouse_led_init(pHW->led_cntr, pHW->gpio_level_led, led_cnt_rate);
#endif

#if MOUSE_DEEPSLEEP_EN
	mouse_sleep_wakeup_init(pStatus->hw_define, pStatus->mouse_sensor);
#endif


}

int mouse_hw_init(void)
{
	mouse_status.hw_define = &mouse_hw;
	mouse_status.data = &mouse_data;
	mouse_status.no_ack = 1;    //no_ack == 0, wakeup sensor from deep-sleep every time

	//gpio_pullup_dpdm_internal( FLD_GPIO_DM_PULLUP | FLD_GPIO_DP_PULLUP );

#if(MOUSE_NO_HW_SIM)

    if(p_custom_cfg->vid == U16_MAX){
    	rf_set_access_code0 (0x39517695);
	}
    else{
    	rf_set_access_code0 (rf_access_code_16to32(p_custom_cfg->vid));
    }
	/*****************   set up did(device id)    ********************/

    if(p_custom_cfg->did != U32_MAX){
        pkt_mouse_pairing.did =  p_custom_cfg->did ;
    }


#if(MOUSE_PIPE1_DATA_WITH_DID)
    pkt_mouse.did = pkt_mouse_pairing.did;
#endif

#if(MOUSE_GOLDEN_VPATTERN)
	extern 	custom_cfg_t   *p_custom_cfg;
	p_custom_cfg = (custom_cfg_t *)(CFG_ADDR_MAC);

	signal_battery = (p_custom_cfg->batt_type == U8_MAX);
    if(p_custom_cfg->vid == U16_MAX){
    	rf_set_access_code0 (0x39517695);
	}
    else{
    	rf_set_access_code0 (rf_access_code_16to32(p_custom_cfg->vid));
    }

    extern rf_packet_pairing_t	pkt_mouse_pairing;
    if(p_custom_cfg->did != U32_MAX){
        pkt_mouse_pairing.did =  p_custom_cfg->did ;
    }

#if(MOUSE_PIPE1_DATA_WITH_DID)
    pkt_mouse.did = pkt_mouse_pairing.did;
#endif

#endif


#else
	mouse_custom_init( &mouse_status );
#endif

	device_info_load(&mouse_status);

#if ( !MOUSE_NO_HW_SIM )
	platform_init(&mouse_status);


#if(MOUSE_SENSOR_MODULE_EN)
#if(DUAL_MOUSE_BOARD != DUAL_MOUSE_MIMOUSE_BOARD)
	mouse_sensor_init(&mouse_status.mouse_sensor, &mouse_status.cpi);
    if ( mouse_status.mouse_sensor & SENSOR_MODE_WORKING ){
        cpu_set_gpio_wakeup(mouse_status.hw_define->sensor_int, 0, 1);  //PAD wakeup enable when wakeup sensor
    	gpio_set_wakeup(mouse_status.hw_define->sensor_int,0,1);  	   //drive pin core(gpio) high wakeup suspend
    }

#if 0	//sdio 1M pull up, sensor sync error
	if(p_custom_cfg->sensor_type == U8_MAX){
		gpio_setup_up_down_resistor(mouse_hw.sensor_data, PM_PIN_PULLUP_1M );
	}
#endif

#endif

#endif

#endif		//end MOUSE_NO_HW_SIM

	return 1;
}
extern led_cfg_t led_cfg[];
void  normal_user_init(void)
{
	//swire2usb_init();
	 //customize freq_offset adjust cap value, if not customized, default ana_81 is 0xd0
	 if( (*(unsigned char*) CUST_CAP_INFO_ADDR_2M) != 0xff ){
		 //ana_81<4:0> is cap value(0x00 - 0x1f)
		 analog_write(0x81, (analog_read(0x81)&0xe0) | ((*(unsigned char*) CUST_CAP_INFO_ADDR_2M)&0x1f) );
	 }

	 // customize TP0/TP1
	 if( ((*(unsigned char*) (CUST_TP_INFO_ADDR_2M)) != 0xff) && ((*(unsigned char*) (CUST_TP_INFO_ADDR_2M+1)) != 0xff) ){
		 rf_update_tp_value(*(unsigned char*) (CUST_TP_INFO_ADDR_2M), *(unsigned char*) (CUST_TP_INFO_ADDR_2M+1));
	 }


    mouse_rf_init(&mouse_status);

#if(DUAL_MOUSE_BOARD != DUAL_MOUSE_MIMOUSE_BOARD)
    device_led_init(GPIO_LED_2P4, 1);
    //gpio_set_output_en(GPIO_LED, 1);
#else
    device_led_init(GPIO_LED_R, 0);
#endif

}


u32 mouse_linked_tick;
void mouse_pair_and_sync_process(mouse_status_t * mouse_status){
    static u16 pairing_time = 0;
    u32 pairing_end = 0;

#if MOUSE_GOLDEN_VPATTERN
    if ( M_HOST_NO_LINK ){
        mouse_status->mouse_mode = STATE_POWERON;
    }

    if ( mouse_status->mouse_mode == STATE_POWERON ) {
        //debug mouse get access code-1 from flash/otp
    	rf_set_access_code1 ( U32_MAX );            //powe-on pkt link fail
    	mouse_status->link = 0;
    	mouse_rf_pkt = (u8 *)&pkt_mouse_pairing;
    	rf_set_tx_pipe(PIPE_PARING);
    	mouse_status->mouse_mode = STATE_SYNCING;
    	if(signal_battery){
    		pkt_mouse_pairing.flow = PKT_FLOW_TOKEN;
    	}
    	else{
    		pkt_mouse_pairing.flow = PKT_FLOW_PARING;
    	}
    }
#endif

    static u8 led_linked_host = 1;      //mouse, wakeup from deep-sleep, can blinky 3 times when linked with dongle

    if ( (mouse_rf_pkt != (u8*)&pkt_mouse) && mouse_status->link ){

    	mouse_linked_tick = clock_time();
        rf_set_tx_pipe (PIPE_MOUSE);

        mouse_rf_pkt = (u8*)&pkt_mouse;						//use data packet
        rf_set_power_level_index (mouse_tx_power);

        mouse_status->mouse_mode = STATE_NORMAL;
        return;
    }

#if DEVICE_LED_MODULE_EN
        //if linked with dongle, led blinky 3 times
        if( led_linked_host && (mouse_status->no_ack == 0) ){
            mouse_led_setup( mouse_led_cfg[E_LED_RSVD] );
            led_linked_host = 0;
        }
#endif

            //device_info_save (mouse_status, 0);

#if(!MOUSE_GOLDEN_VPATTERN)

        if ( mouse_status->mouse_mode == STATE_PAIRING ){
        	pkt_mouse_pairing.flow = PKT_FLOW_PARING;
        }
        else{
        	pkt_mouse_pairing.flow = PKT_FLOW_TOKEN;
        	if(mouse_status->loop_cnt >= PARING_POWER_ON_CNT ){
        		mouse_status->mouse_mode = STATE_SYNCING;		//SYNCING
        	}
        }
#endif


#if 1
    {
        if ( mouse_status->mouse_mode == STATE_PAIRING ){
            rf_set_power_level_index ( mouse_tx_paring_power);
        }
        else if ( (mouse_status->mouse_mode <= STATE_PAIRING) && (mouse_status->loop_cnt >= PARING_POWER_ON_CNT) ){
            rf_set_power_level_index (mouse_tx_power);
        }
    }
#endif

}
//u32 debug_last_wakeup_level;
extern unsigned short battValue[10];
extern void mouse_battery_check(unsigned short *batt, u8 len);
#if 1
_attribute_ram_code_
#endif
void mouse_task_when_rf ( void ){


	*((u32 *)(mouse_status.data)) = 0;	//clear mouse_event_data
#if(MOUSE_WHEEL_MODULE_EN)
    u32 wheel_prepare_tick = mouse_wheel_prepare_tick();
#endif



#if MOUSE_NO_HW_SIM

    if ( 1 || MOUSE_V_PATERN_MODE || (mouse_status.dbg_mode & STATE_TEST_V_BIT) ){
        static u8 fno;
        mouse_status.data->x = fno & BIT(6) ? 6 : -6;
        mouse_status.data->y = fno & BIT(5) ? -6 : 6;
        fno++;
    }
    mouse_pair_and_sync_process(&mouse_status);
    mouse_power_saving_process(&mouse_status);
#else


#if(MOUSE_WHEEL_MODULE_EN)
    mouse_wheel_process(&mouse_status, wheel_prepare_tick);
#endif


#if(MOUSE_BTN_MODULE_EN)
    mouse_button_process(&mouse_status);
#endif
    
#if(MOUSE_SLEEP_MODULE_EN)
    mouse_power_saving_process(&mouse_status);
#endif

#endif
}

extern u8	device_channel;
extern u32  ll_chn_mask;
void mouse_task_in_ram( void ){

    p_task_when_rf = mouse_task_when_rf;
    mouse_rf_process(&mouse_status);

    device_channel = get_next_channel_with_mask (ll_chn_mask, device_channel);

    if (p_task_when_rf != NULL) {

       (*p_task_when_rf) ();
    }
}


void mouse_main_loop(void)
{

	if( MOUSE_EMI_4_FCC || (mouse_status.mouse_mode == STATE_EMI) ){
		mouse_emi_process(&mouse_status);
	}
	else
	{
		mouse_sleep.wakeup_tick = clock_time();

#if(!MOUSE_NO_HW_SIM)
        if(mouse_status.mouse_mode <= STATE_PAIRING){				//pair and sync
        	mouse_pair_and_sync_process(&mouse_status);
        }

#if(DUAL_MOUSE_BOARD == DUAL_MOUSE_TELINK_BOARD_V1P0)
        mouse_button_detect(&mouse_status, MOUSE_BTN_LOW);		//Debug, no button

#elif(DUAL_MOUSE_BOARD == DUAL_MOUSE_TELINK_BOARD_V2P0)

        mouse_button_pull(&mouse_status, MOUSE_BTN_LOW);
        mouse_button_detect(&mouse_status, MOUSE_BTN_HIGH);		//Debug, no button

#endif

#if(MOUSE_SENSOR_MODULE_EN)
#if(DUAL_MOUSE_BOARD != DUAL_MOUSE_MIMOUSE_BOARD)
        	mouse_sensor_data( &mouse_status );
#else
        drv_mouse_motion_report(&mouse_status.data->x, 0);			//Debug, no sensor
#endif

#endif

#if(MOUSE_BATT_MOUDULE_EN)
        mouse_battery_check(battValue, sizeof(battValue)/sizeof(battValue[0]));
#endif

#if(DUAL_MOUSE_BOARD == DUAL_MOUSE_TELINK_BOARD_V2P0)
        mouse_button_pull(&mouse_status, MOUSE_BTN_HIGH);
        mouse_button_detect(&mouse_status, MOUSE_BTN_LOW);		//Debug, no button
#endif

#endif
        mouse_task_in_ram();

        device_led_process();										//Debug, no led
        int wakeup_status = cpu_sleep_wakeup( mouse_sleep.mode == SLEEP_MODE_DEEPSLEEP, mouse_sleep.wakeup_src, mouse_sleep.wakeup_next_tick );
		
    	if(!(wakeup_status & 2)){  // if not timer wakeup, need re_sync chn
    		device_sync = 0;
    	}
	}
    mouse_status.loop_cnt ++;
}

