#include "../../proj/tl_common.h"
#include "../../proj/mcu/watchdog_i.h"
#include "../../proj_lib/rf_drv.h"
#include "../../proj_lib/pm.h"
#include "../common/rf_frame.h"
#include "mouse_info.h"
#include "mouse.h"
#include "mouse_button.h"
#include "mouse_custom.h"
#include "../../proj_lib/ble/ll/ll.h"

#define			PM_REG_START		0x19
#define			PM_REG_END			0x1f

#define			DEVICE_INFO_STORE	1

#ifndef PM_POWERON_DETECTION_ENABLE
#define PM_POWERON_DETECTION_ENABLE 0
#endif


static device_rsv_info_t device_info;

/*
 *  Base on from power on or deep sleep back
 *  Load customizable information from the 3.3V Analog register
 *  or Load from OTP
 *
 */

#if 1

void device_info_load(mouse_status_t *mouse_status)
{
#if DEVICE_INFO_STORE
    u8 * pd = (u8 *) &device_info;
    device_info.mode = analog_read(DEEP_ANA_REG5);

	device_info.dpi = analog_read(DEEP_ANA_REG7);
    mouse_status->mouse_mode = (device_info.mode >> 2) & 0x3 ;

    if(button_func.system_mode == RF_2M_2P4G_MODE){
    	for (int i=DEEP_ANA_REG0; i<=DEEP_ANA_REG3; i++) {
    		*pd ++ = analog_read (i);
    	}

        if ( mouse_status->mouse_mode == STATE_NORMAL ){

        	mouse_status->mouse_sensor = device_info.mode & INFO_SENSOR_STATUS_CTRL;
            mouse_status->cpi = device_info.mode & INFO_SENSOR_CPI_CTRL & 0x3;
        	mouse_status->dongle_id = device_info.dongle_id;
            rf_set_access_code1 (mouse_status->dongle_id);
        }
        else{
        	mouse_status->cpi = mouse_cpi.sns_cpi_dft;
        }

    }
    else if(button_func.system_mode == RF_1M_BLE_MODE){
    	mouse_status->mouse_sensor = device_info.mode & INFO_SENSOR_STATUS_CTRL;
    	/*if(mouse_status->mouse_mode == STATE_NORMAL){							//the cpi has been saved
        	mouse_status->cpi = device_info.mode & INFO_SENSOR_CPI_CTRL & 0x3;

    	}
    	else if(mouse_status->mouse_mode == STATE_POWERON){
			
    		mouse_status->cpi = mouse_cpi.sns_cpi_dft;							//use default cpi
    	}*/
    	if((device_info.dpi == 1) || (device_info.dpi == 2) || (device_info.dpi == 3) || (device_info.dpi == 4)){
			mouse_status->cpi = device_info.dpi - 1;
		}
		else{
			mouse_status->cpi = mouse_cpi.sns_cpi_dft;
		}


    }
   //mouse_status->cpi = 1;			//default = 1

//   Need get poweron, cpi, etc back first

#else
    mouse_status->mouse_mode = device_info.mode ? STATE_NORMAL : STATE_POWERON;
#endif
}

/*
 * Save the information need from the deep sleep back
 *
 */
//extern button_func_t button_func;
void device_info_save(mouse_status_t *mouse_status, u32 sleep_save)
{

    u8 * pd = (u8 *) &device_info;

    device_info.sensor = (mouse_status->mouse_sensor & 0xf0) |  (mouse_status->cpi & 0x0f);

    if(button_func.system_mode == RF_2M_2P4G_MODE){			//2.4G mode

    	//device_info.mode |= (mouse_status->mouse_sensor & 0xf0) ;
    	//device_info.sensor = (mouse_status->mouse_sensor & 0xf0) | (mouse_status->cpi & 0x0f);
    	device_info.mode = device_info.sensor |  (mouse_status->mouse_mode << 2);
    	//device_info.mode = mouse_status->mouse_mode;

    	device_info.dongle_id = mouse_status->dongle_id;
    	for (u8 i=DEEP_ANA_REG0; i<=DEEP_ANA_REG3; i++) {
    		analog_write (i, *pd ++);
    	}
        analog_write(DEEP_ANA_REG5, device_info.mode);

    }
    else if(button_func.system_mode == RF_1M_BLE_MODE){

    	u8 ble_state = analog_read(DEEP_ANA_REG5) & 0x0c;
    	device_info.mode = device_info.sensor |  (ble_state << 2);
    	analog_write(DEEP_ANA_REG5, device_info.mode);

		device_info.dpi = mouse_status->cpi +1;
		analog_write(DEEP_ANA_REG7,device_info.dpi);
    }



}

#else
extern u8 dual_mouse_state;
void device_info_load(mouse_status_t *mouse_status)
{
#if DEVICE_INFO_STORE
    u8 * pd = (u8 *) &device_info;
    device_info.mode = analog_read(DEEP_ANA_REG4);
    device_info.sensor = analog_read(DEEP_ANA_REG6);


    if(button_func.system_mode == RF_2M_2P4G_MODE){
    	for (int i=DEEP_ANA_REG0; i<=DEEP_ANA_REG3; i++) {
    		*pd ++ = analog_read (i);
    	}

    	mouse_status->mouse_mode = (device_info.mode  & 0x0f) ;

        if ( mouse_status->mouse_mode == STATE_NORMAL ){

        	mouse_status->mouse_sensor = device_info.sensor & INFO_SENSOR_STATUS_CTRL;
            mouse_status->cpi = device_info.sensor & INFO_SENSOR_CPI_CTRL;
        	mouse_status->dongle_id = device_info.dongle_id;
            rf_set_access_code1 (mouse_status->dongle_id);

        }

    }
    else if(button_func.system_mode == RF_1M_BLE_MODE){
    	u8 ble_state = ((device_info.mode & 0xf0) >> 4) ;
    	if(ble_state == CONN_DEEP_FLG){
    		mouse_status->mouse_sensor = device_info.sensor & INFO_SENSOR_STATUS_CTRL;
    		mouse_status->cpi = device_info.sensor & INFO_SENSOR_CPI_CTRL;
    	}
    }



    //mouse_status->cpi = 1;			//default = 1

    //mouse_status->mouse_mode = device_info.mode & 0x03;

//   Need get poweron, cpi, etc back first

#else
    mouse_status->mouse_mode = device_info.mode ? STATE_NORMAL : STATE_POWERON;
#endif
}

/*
 * Save the information need from the deep sleep back
 *
 */
//extern button_func_t button_func;
void device_info_save(mouse_status_t *mouse_status, u32 sleep_save)
{

    u8 * pd = (u8 *) &device_info;

    u8 status = analog_read(DEEP_ANA_REG4);

    device_info.sensor = (mouse_status->mouse_sensor & 0xf0) |  (mouse_status->cpi & 0x0f);

    if(button_func.system_mode == RF_2M_2P4G_MODE){			//2.4G mode

    	//device_info.sensor = (mouse_status->mouse_sensor & 0xf0) | (mouse_status->cpi & 0x0f);
    	//device_info.mode = device_info.sensor |  (mouse_status->mouse_mode << 2);
    	device_info.mode = mouse_status->mouse_mode | (status & 0xf0);

    	device_info.dongle_id = mouse_status->dongle_id;
    	for (u8 i=DEEP_ANA_REG0; i<=DEEP_ANA_REG4; i++) {
    		analog_write (i, *pd ++);
    	}


    }
    else if(button_func.system_mode == RF_1M_BLE_MODE){

    	device_info.mode = (dual_mouse_state << 4) |  (status & 0x0f);
    	analog_write(DEEP_ANA_REG4, device_info.mode);

    }
    analog_write(DEEP_ANA_REG6, device_info.sensor);


}
#endif


