#include "../../proj/tl_common.h"
#include "../../proj_lib/ble/ll/ll.h"
#include "../../proj_lib/ble/service/ble_ll_ota.h"

 u8 FwVersion[5]={((PRJ_NAME<<8)&0xff),(PRJ_NAME&0xff),2,0,1};

#define HID_REPORT_ID_KEYBOARD_INPUT_AAA            1 //!< Keyboard input report ID
#define HID_REPORT_ID_CONSUME_CONTROL_INPUT_AAA     2//!< Consumer Control input report ID
#define HID_REPORT_ID_MOUSE_INPUT_AAA				3//!< Mouse input report ID
#define HID_REPORT_ID_GAMEPAD_INPUT_AAA				4 //!< Gamepad  input   
#define HID_REPORT_ID_SYSTEMP_INPUT_AAA				5 //!< Gamepad  input 

typedef struct
{
  /** Minimum value for the connection event (interval. 0x0006 - 0x0C80 * 1.25 ms) */
  u16 intervalMin;
  /** Maximum value for the connection event (interval. 0x0006 - 0x0C80 * 1.25 ms) */
  u16 intervalMax;
  /** Number of LL latency connection events (0x0000 - 0x03e8) */
  u16 latency;
  /** Connection Timeout (0x000A - 0x0C80 * 10 ms) */
  u16 timeout;
} gap_periConnectParams_t;

const u16 clientCharacterCfgUUID = GATT_UUID_CLIENT_CHAR_CFG;

const u16 extReportRefUUID = GATT_UUID_EXT_REPORT_REF;

const u16 reportRefUUID = GATT_UUID_REPORT_REF;

const u16 characterPresentFormatUUID = GATT_UUID_CHAR_PRESENT_FORMAT;

const u16 my_primaryServiceUUID = GATT_UUID_PRIMARY_SERVICE;

static const u16 my_characterUUID = GATT_UUID_CHARACTER;

const u16 my_devServiceUUID = SERVICE_UUID_DEVICE_INFORMATION;

const u16 my_PnPUUID = CHARACTERISTIC_UUID_PNP_ID;


const u16 my_devNameUUID = GATT_UUID_DEVICE_NAME;

//device information
const u16 my_gapServiceUUID = SERVICE_UUID_GENERIC_ACCESS;
// Appearance Characteristic Properties
const u16 my_appearanceUIID = 0x2a01;
const u16 my_periConnParamUUID = 0x2a04;
//u16 my_appearance = GAP_APPEARE_UNKNOWN;
u8 my_appearance[2] = {0xc1, 0x03};
gap_periConnectParams_t my_periConnParameters = {8, 16, 4, 600};


const u16 my_gattServiceUUID = SERVICE_UUID_GENERIC_ATTRIBUTE;
const u16 serviceChangeUIID = GATT_UUID_SERVICE_CHANGE;
u16 serviceChangeVal[2] = {0};
static u8 serviceChangeCCC[2]={0,0};



const u8 PROP_READ = CHAR_PROP_READ;

const u8 PROP_WRITE = CHAR_PROP_WRITE;

const u8 PROP_INDICATE = CHAR_PROP_INDICATE;

const u8 PROP_WRITE_NORSP = CHAR_PROP_WRITE_WITHOUT_RSP;

const u8 PROP_READ_NOTIFY = CHAR_PROP_READ | CHAR_PROP_NOTIFY;

const u8 PROP_READ_WRITE_NORSP = CHAR_PROP_READ | CHAR_PROP_WRITE_WITHOUT_RSP;

const u8 PROP_READ_WRITE_WRITENORSP = CHAR_PROP_READ | CHAR_PROP_WRITE | CHAR_PROP_WRITE_WITHOUT_RSP;

const u8 PROP_READ_WRITE = CHAR_PROP_READ | CHAR_PROP_WRITE;

const u8 PROP_READ_WRITE_NORSP_NOTIFY = CHAR_PROP_READ | CHAR_PROP_WRITE_WITHOUT_RSP | CHAR_PROP_NOTIFY;



const u8	 my_PnPtrs [7] = {0x02, 0x8a, 0x24, 0x66, 0x82, 0x01, 0x00};//telink
//const u8	 my_PnPtrs [] = {0x02, 0xac, 0x05, 0x20, 0x02, 0x01, 0x00};//nordic keyboard
//const u8	my_PnPtrs [] = {0x02, 0xac, 0x05, 0x2c, 0x02, 0x01, 0x00};//apple
//const u8	my_PnPtrs [] = {0x02, 0xac, 0x05, 0x4f, 0x02, 0x01, 0x00};//apple



//////////////////////// Battery /////////////////////////////////////////////////
const u16 my_batServiceUUID       			= SERVICE_UUID_BATTERY;
const u16 my_batCharUUID       				= CHARACTERISTIC_UUID_BATTERY_LEVEL;
static u8 batteryValueInCCC[2];
u8 		  my_batVal[1] 						= {99};

//////////////////////// HID /////////////////////////////////////////////////////

const u16 my_hidServiceUUID       			= SERVICE_UUID_HUMAN_INTERFACE_DEVICE;

const u16 hidServiceUUID           = SERVICE_UUID_HUMAN_INTERFACE_DEVICE;
const u16 hidProtocolModeUUID      = CHARACTERISTIC_UUID_HID_PROTOCOL_MODE;
const u16 hidReportUUID            = CHARACTERISTIC_UUID_HID_REPORT;
const u16 hidReportMapUUID         = CHARACTERISTIC_UUID_HID_REPORT_MAP;
const u16 hidbootKeyInReportUUID   = CHARACTERISTIC_UUID_HID_BOOT_KEY_INPUT;
const u16 hidbootKeyOutReportUUID  = CHARACTERISTIC_UUID_HID_BOOT_KEY_OUTPUT;
const u16 hidbootMouseInReportUUID = CHARACTERISTIC_UUID_HID_BOOT_MOUSE_INPUT;
const u16 hidinformationUUID       = CHARACTERISTIC_UUID_HID_INFORMATION;
const u16 hidCtrlPointUUID         = CHARACTERISTIC_UUID_HID_CONTROL_POINT;
const u16 hidIncludeUUID           = GATT_UUID_INCLUDE;

u8 protocolMode = DFLT_HID_PROTOCOL_MODE;


// Key in Report characteristic variables
u8 reportKeyIn[8];
u8 reportKeyInCCC[2];
// HID Report Reference characteristic descriptor, key input
static u8 reportRefKeyIn[2] =
             { HID_REPORT_ID_KEYBOARD_INPUT_AAA, HID_REPORT_TYPE_INPUT };

// Key out Report characteristic variables
u8 reportKeyOut=0;
u8 reportKeyOutCCC[2];
static u8 reportRefKeyOut[2] =
             { HID_REPORT_ID_KEYBOARD_INPUT_AAA, HID_REPORT_TYPE_OUTPUT };

// Consumer Control input Report
static u8 reportConsumerControlIn[4];
static u8 reportConsumerControlInCCC[2];
static u8 reportRefConsumerControlIn[2] = 
			{ HID_REPORT_ID_CONSUME_CONTROL_INPUT_AAA, HID_REPORT_TYPE_INPUT };

// Consumer Control input Report
static u8 reportSystemIn[1];
static u8 reportSystemInCCC[2];
static u8 reportRefSystemIn[2] = 
			{ HID_REPORT_ID_SYSTEMP_INPUT_AAA, HID_REPORT_TYPE_INPUT };

// HID Information characteristic
const u8 hidInformation[] =
{
  U16_LO(0x0111), U16_HI(0x0111),             // bcdHID (USB HID version)
  0x00,                                       // bCountryCode
  0x01                                        // Flags
};

// HID Control Point characteristic
static u8 controlPoint;

// HID Report Map characteristic
// Keyboard report descriptor (using format for Boot interface descriptor)

static const u8 reportMap[] =
{
	//keyboard report in
	0x05, 0x01,     // Usage Pg (Generic Desktop)
	0x09, 0x06,     // Usage (Keyboard)
	0xA1, 0x01,     // Collection: (Application)
	0x85, HID_REPORT_ID_KEYBOARD_INPUT_AAA,     // Report Id (keyboard)
				  //
	0x05, 0x07,     // Usage Pg (Key Codes)
	0x19, 0xE0,     // Usage Min (224)  VK_CTRL:0xe0
	0x29, 0xE7,     // Usage Max (231)  VK_RWIN:0xe7
	0x15, 0x00,     // Log Min (0)
	0x25, 0x01,     // Log Max (1)
				  //
				  // Modifier byte
	0x75, 0x01,     // Report Size (1)   1 bit * 8
	0x95, 0x08,     // Report Count (8)
	0x81, 0x02,     // Input: (Data, Variable, Absolute)
				  //
				  // Reserved byte
	0x95, 0x01,     // Report Count (1)
	0x75, 0x08,     // Report Size (8)
	0x81, 0x01,     // Input: (Constant)

	//keyboard output
	//5 bit led ctrl: NumLock CapsLock ScrollLock Compose kana
	0x95, 0x05,    //Report Count (5)
	0x75, 0x01,    //Report Size (1)
	0x05, 0x08,    //Usage Pg (LEDs )
	0x19, 0x01,    //Usage Min
	0x29, 0x05,    //Usage Max
	0x91, 0x02,    //Output (Data, Variable, Absolute)
	//3 bit reserved
	0x95, 0x01,    //Report Count (1)
	0x75, 0x03,    //Report Size (3)
	0x91, 0x01,    //Output (Constant)

	// Key arrays (6 bytes)
	0x95, 0x06,     // Report Count (6)
	0x75, 0x08,     // Report Size (8)
	0x15, 0x00,     // Log Min (0)
	0x25, 0xF1,     // Log Max (241)
	0x05, 0x07,     // Usage Pg (Key Codes)
	0x19, 0x00,     // Usage Min (0)
	0x29, 0xf1,     // Usage Max (241)
	0x81, 0x00,     // Input: (Data, Array)

	0xC0,            // End Collection


#if CONSUMER_REPORT_BYTE_AAA

	//consumer report in
	0x05, 0x0C,   // Usage Page (Consumer)
	0x09, 0x01,   // Usage (Consumer Control)
	0xA1, 0x01,   // Collection (Application)
	0x85, HID_REPORT_ID_CONSUME_CONTROL_INPUT_AAA,   //     Report Id
	0x75,0x10,     //global, report size 16 bits
	0x95,0x01,     //global, report count 1
	0x15,0x01,     //global, min  0x01
	0x26,0x8c,0x02,  //global, max  0x28c
	0x19,0x01,     //local, min   0x01
	0x2a,0x8c,0x02,  //local, max    0x28c
	0x81,0x00,     //main,  input data varible, absolute
	0xc0,        //main, end collection
#else

	0x05,0x0C,		// USAGE PAGE (CONSUMER PAGE)
	0x09,0x01,			// USAGE (CONSUMER CONTROL)
	0xA1,0x01,			// COLLECTION (APPLICATION)

	0x85,HID_REPORT_ID_CONSUME_CONTROL_INPUT_AAA,		//	REPORT_ID (CONSUMER)

	0x15,0x00,			// LOGICAL_MINIMUM(0)
	0x25,0x01,			// LOGICAL_MAXIMUM(1)
	0x75,0x01,			// REPORT_SIZE(1)


	0x0A,0x21,0x02,		//0xa3 C_WWW_SEARCH		
	0x0A,0x23,0x02,		//0xa4 C_WWW_HOME
	0x0A,0x24,0x02,		//0xa5 C_WWW_BACKWARD
	0x0A,0x25,0x02,		//0xa6 C_WWW_FORWARD
	0x0A,0x26,0x02,		//0xa7 C_WWW_STOP
	0x0A,0x27,0x02,		//0xa8 C_WWW_REFRESH
	0x0A,0x2A,0x02,		//0xa9 C_MY_FAVORITE
	0x0A,0x83,0x01,		//0xaa C_MEDIA_SELECT
	
	0x0A,0x8A,0x01,		//0xab C_EMAIL
	0x0A,0x92,0x01,		//0xac C_CALCULATOR
	0x0A,0x94,0x01,		//0xad C_MY_COMPUTER
	0x0A,0xB5,0x00,		//0xae C_NEXT_TRACK
	0x0A,0xB6,0x00,		//0xaf C_PRE_TRACK
	0x0A,0xB7,0x00,		//0xb0 C_STOP
	0x0A,0xCD,0x00,		//0xb1 C_PLAY_PAUSE
	0x0A,0xE2,0x00,		//0xb2 C_MUTE	
	
	0x0A,0xE9,0x00,		//0xb3 C_VOL_INC
	0x0A,0xEA,0x00,		//0xb4 C_VOL_DEC	
	0x0A,0x00,0x00,		//0xb5 telinkԶ
	0x0A,0x2D,0x02,		//0xb6 USAGE ZOOM IN
	0x0A,0x2E,0x02,		//0xb7	 USAGE ZOOM OUT 
	0x0A,0x36,0x02,		//0xb8	 USAGE PAN LEFT
	0x0A,0x37,0x02,		//0xb9	 USAGE PAN RIGHT
	0x0A,0x0B,0x03,		//0xba	C_BRIGHT_INC	
	
	0x0A,0x0A,0x03,		//0xbb	C_BRIGHT_DEC
	0x0A,0xB8,0X00,		//0Xbc	 c_rject
	0x0A,0x30,0x00,		//0Xbd	C_POWER 		
	0x0A,0x9E,0x01,		//0Xbe	C_TERMINAL_LOCK 

	0x95,28,			//	REPORT_COUNT (8)
	0x81,0x02,			//	INPUT()

	0x95,4,			//	REPORT_COUNT (8)
	0x81, 0x01, 	// Input: (Constant)


	0xC0,				// END COLLECTION   

#endif

#if SYSTEM_KEY_FUN_ENABLE_AAA

0x05,0x01, 		//Usage Page (Generic Desktop Control)
0x09,0x80,			//Usage (SYSTEM CONTROL)
0xA1,0x01,	//Collection (Application)

0x85,HID_REPORT_ID_SYSTEMP_INPUT_AAA,		//Report ID (ACPI)

0x25,0x01,			//	Logical Maximum (1)
0x15,0x00,			//	Logical Minimum (0)
0x75,0x01,			//	Report Size


0x09,0x82, 	//	USAGE SYSTEM SLEEP
0x09,0x81,	//	USAGE SYSTEM POWER DOWN 
0x09,0x83, 	//	USAGE SYSTEM WAKE UP

0x95,0x03,			//	REPORT_COUNT (03H)
0x81,0x02,			//	INPUT (DATA, VAR)								 
0x95,0x05,			//	Report Count (05)

0x81,0x01,			//	Input (CONSTANT)
0xC0,				//	END COLLECTION
#endif

};

// HID External Report Reference Descriptor for report map
//static u16 extServiceUUID;


/////////////////////////////////////////////////////////
const u8 my_AudioUUID[16]   = TELINK_AUDIO_UUID_SERVICE;


const u8 my_OtaUUID[16]		= TELINK_SPP_DATA_OTA;
const u8 my_OtaServiceUUID[16]		= TELINK_OTA_UUID_SERVICE;
const u16 userdesc_UUID		= GATT_UUID_CHAR_USER_DESC;
const u8 my_fwUUID[16]		= {0x79,0x61,0x6E,0x67,0x6B,0x65,0x71,0x69,0x79,0x61,0x6E,0x67,0x6B,0x65,0x71,0x69};

u8	 	my_OtaData 		= 0x00;






// Include attribute (Battery service)
//static u16 include[3] = {0x2a, 0x2d, SERVICE_UUID_BATTERY};

extern int kb_led_out_write_aaa(void* p);
extern u8  ble_devName[];

// TM : to modify
const attribute_t my_Attributes[] = {

	{4+7+3+25+4+5, 0,0,0,0,0},	// total num of attribute

	// 0008 - 000b gatt
	{4,ATT_PERMISSIONS_READ,2,2,(u8*)(&my_primaryServiceUUID),	(u8*)(&my_gattServiceUUID), 0},
	{0,ATT_PERMISSIONS_READ,2,1,(u8*)(&my_characterUUID),		(u8*)(&PROP_INDICATE), 0},
	{0,ATT_PERMISSIONS_READ,2,sizeof (serviceChangeVal), (u8*)(&serviceChangeUIID), 	(u8*)(&serviceChangeVal), 0},
	{0,ATT_PERMISSIONS_RDWR,2,sizeof (serviceChangeCCC),(u8*)(&clientCharacterCfgUUID), (u8*)(serviceChangeCCC), 0},

	// 0001 - 0007  gap
	{7,ATT_PERMISSIONS_READ,2,2,(u8*)(&my_primaryServiceUUID), 	(u8*)(&my_gapServiceUUID), 0},
	{0,ATT_PERMISSIONS_READ,2,1,(u8*)(&my_characterUUID), 		(u8*)(&PROP_READ_NOTIFY), 0},
	{0,ATT_PERMISSIONS_READ,2,MAX_DEV_NAME_LEN, (u8*)(&my_devNameUUID), (u8*)(&ble_devName), 0},
	{0,ATT_PERMISSIONS_READ,2,1,(u8*)(&my_characterUUID), 		(u8*)(&PROP_READ), 0},
	{0,ATT_PERMISSIONS_READ,2,sizeof (my_appearance), (u8*)(&my_appearanceUIID), 	(u8*)(&my_appearance), 0},
	{0,ATT_PERMISSIONS_READ,2,1,(u8*)(&my_characterUUID), 		(u8*)(&PROP_READ), 0},
	{0,ATT_PERMISSIONS_READ,2,sizeof (my_periConnParameters),(u8*)(&my_periConnParamUUID), 	(u8*)(&my_periConnParameters), 0},

	// 000c - 000e  device Information Service
	{3,ATT_PERMISSIONS_READ,2,2,(u8*)(&my_primaryServiceUUID), 	(u8*)(&my_devServiceUUID), 0},
	{0,ATT_PERMISSIONS_READ,2,1,(u8*)(&my_characterUUID), 		(u8*)(&PROP_READ), 0},
	{0,ATT_PERMISSIONS_READ,2,sizeof (my_PnPtrs),(u8*)(&my_PnPUUID), (u8*)(my_PnPtrs), 0},
	

	/////////////////////////////////// 4. HID Service /////////////////////////////////////////////////////////
	// 000f
	//{27, ATT_PERMISSIONS_READ,2,2,(u8*)(&my_primaryServiceUUID), 	(u8*)(&my_hidServiceUUID), 0},
	{25, ATT_PERMISSIONS_READ,2,2,(u8*)(&my_primaryServiceUUID), 	(u8*)(&my_hidServiceUUID), 0},

	// 0010  include battery service
	//{0,ATT_PERMISSIONS_READ,2,sizeof(include),(u8*)(&hidIncludeUUID), 	(u8*)(include), 0},

	// 0011 - 0012  protocol mode
	{0,ATT_PERMISSIONS_READ,2,1,(u8*)(&my_characterUUID), 		(u8*)(&PROP_READ_WRITE_NORSP), 0},				//prop
	{0,ATT_PERMISSIONS_RDWR,2, sizeof(protocolMode),(u8*)(&hidProtocolModeUUID), 	(u8*)(&protocolMode), 0},	//value


	// 0018 - 001b. consume report in: 4 (char-val-client-ref)
	{0,ATT_PERMISSIONS_READ,2,1,(u8*)(&my_characterUUID), 		(u8*)(&PROP_READ_NOTIFY), 0},				//prop
	{0,ATT_PERMISSIONS_READ,2, sizeof(reportConsumerControlIn),(u8*)(&hidReportUUID), 	(u8*)(reportConsumerControlIn), 0},	//value
	{0,ATT_PERMISSIONS_RDWR,2,sizeof(reportConsumerControlInCCC),(u8*)(&clientCharacterCfgUUID), 	(u8*)(reportConsumerControlInCCC), 0},	//value
	{0,ATT_PERMISSIONS_RDWR,2,sizeof(reportRefConsumerControlIn),(u8*)(&reportRefUUID), 	(u8*)(reportRefConsumerControlIn), 0},	//value

	// 001c - 001f . keyboard report in : 4 (char-val-client-ref)
	{0,ATT_PERMISSIONS_READ,2,1,(u8*)(&my_characterUUID), 		(u8*)(&PROP_READ_NOTIFY), 0},				//prop
	{0,ATT_PERMISSIONS_READ,2, sizeof(reportKeyIn),(u8*)(&hidReportUUID), 	(u8*)(reportKeyIn), 0},	//value
	{0,ATT_PERMISSIONS_RDWR,2,sizeof(reportKeyInCCC),(u8*)(&clientCharacterCfgUUID), 	(u8*)(reportKeyInCCC), 0},	//value
	{0,ATT_PERMISSIONS_RDWR,2,sizeof(reportRefKeyIn),(u8*)(&reportRefUUID), 	(u8*)(reportRefKeyIn), 0},	//value

	// 0020 - 0022 . keyboard report out: 3 (char-val-ref)
	{0,ATT_PERMISSIONS_READ,2,1,(u8*)(&my_characterUUID), 		(u8*)(&PROP_READ_WRITE_WRITENORSP), 0},				//prop
	{0,ATT_PERMISSIONS_RDWR,2,sizeof(reportKeyOut),(u8*)(&hidReportUUID), 	(u8*)(&reportKeyOut), &kb_led_out_write_aaa},	//value
	{0,ATT_PERMISSIONS_RDWR,2,sizeof(reportRefKeyOut),(u8*)(&reportRefUUID), 	(u8*)(reportRefKeyOut), 0},	//value

	// 001c - 001f . keyboard report in : 4 (char-val-client-ref)
	{0,ATT_PERMISSIONS_READ,2,1,(u8*)(&my_characterUUID),		(u8*)(&PROP_READ_NOTIFY), 0},				//prop
	{0,ATT_PERMISSIONS_READ,2, sizeof(reportSystemIn),(u8*)(&hidReportUUID),	(u8*)(reportSystemIn), 0}, //value
	{0,ATT_PERMISSIONS_RDWR,2,sizeof(reportSystemInCCC),(u8*)(&clientCharacterCfgUUID),	(u8*)(reportSystemInCCC), 0},	//value
	{0,ATT_PERMISSIONS_RDWR,2,sizeof(reportRefSystemIn),(u8*)(&reportRefUUID), 	(u8*)(reportRefSystemIn), 0},	//value

	// 0023 - 0025 . report map: 3
	{0,ATT_PERMISSIONS_READ,2,1,(u8*)(&my_characterUUID), 		(u8*)(&PROP_READ), 0},				//prop
	{0,ATT_PERMISSIONS_AUTHEN_READ,2,sizeof(reportMap),(u8*)(&hidReportMapUUID), 	(u8*)(reportMap), 0},	//value
	//{0,ATT_PERMISSIONS_RDWR,2,sizeof(extServiceUUID),(u8*)(&extReportRefUUID), 	(u8*)(&extServiceUUID), 0},	//value

	// 0026 - 0027 . hid information: 2
	{0,ATT_PERMISSIONS_READ,2,1,(u8*)(&my_characterUUID), 		(u8*)(&PROP_READ), 0},				//prop
	{0,ATT_PERMISSIONS_READ,2, sizeof(hidInformation),(u8*)(&hidinformationUUID), 	(u8*)(hidInformation), 0},	//value

	// 0028 - 0029 . control point: 2
	{0,ATT_PERMISSIONS_READ,2,1,(u8*)(&my_characterUUID), 		(u8*)(&PROP_WRITE_NORSP), 0},				//prop
	{0,ATT_PERMISSIONS_WRITE,2, sizeof(controlPoint),(u8*)(&hidCtrlPointUUID), 	(u8*)(&controlPoint), 0},	//value

	////////////////////////////////////// Battery Service /////////////////////////////////////////////////////
	// 002a - 002d
	{4,ATT_PERMISSIONS_READ,2,2,(u8*)(&my_primaryServiceUUID), 	(u8*)(&my_batServiceUUID), 0},
	{0,ATT_PERMISSIONS_READ,2,1,(u8*)(&my_characterUUID), 		(u8*)(&PROP_READ_NOTIFY), 0},				//prop
	{0,ATT_PERMISSIONS_READ,2,sizeof(my_batVal),(u8*)(&my_batCharUUID), 	(u8*)(my_batVal), 0},	//value
	{0,ATT_PERMISSIONS_RDWR,2,sizeof(batteryValueInCCC),(u8*)(&clientCharacterCfgUUID), 	(u8*)(batteryValueInCCC), 0},	//value

	////////////////////////////////////// OTA /////////////////////////////////////////////////////
	// 002e - 0031
	{5,ATT_PERMISSIONS_READ, 2,16,(u8*)(&my_primaryServiceUUID), 	(u8*)(&my_OtaServiceUUID), 0},
	{0,ATT_PERMISSIONS_READ, 2, 1,(u8*)(&my_characterUUID), 		(u8*)(&PROP_READ_WRITE_NORSP), 0},				//prop
	{0,ATT_PERMISSIONS_RDWR,16,sizeof(my_OtaData),(u8*)(&my_OtaUUID),	(&my_OtaData), &otaWrite, &otaRead},			//value
	//{0,ATT_PERMISSIONS_READ, 2,sizeof (my_OtaName),(u8*)(&userdesc_UUID), (u8*)(my_OtaName), 0},
	
	{0,ATT_PERMISSIONS_READ,2,1,(u8*)(&my_characterUUID), 	(u8*)(&PROP_READ), 0},
	{0,ATT_PERMISSIONS_READ,16,sizeof(FwVersion),(u8*)(&my_fwUUID), (u8*)(FwVersion), 0},
};

void	my_att_init ()
{
	bls_att_setAttributeTable ((u8 *)my_Attributes);
}

