#ifndef _AAA_CONFIG_H_
#define _AAA_CONFIG_H_

#include "../../proj/tl_common.h"
#include "../../proj_lib/rf_drv.h"
#include "../../proj_lib/pm.h"
#include "../../proj_lib/ble/ll/ll.h"
#include "../../proj_lib/ble/trace.h"
#include "../../proj_lib/ble/service/ble_ll_ota.h"
#include "../../proj/drivers/adc.h"
#include "../../proj/drivers/battery.h"
#include "../../proj_lib/ble/blt_config.h"
#include "../../proj_lib/ble/ble_smp.h"
#include "../../proj/drivers/keyboard.h"
#include "../common/blt_led.h"
#include "../../proj_lib/ble/ble_common.h"

#include "aaa_rf_frame.h"
#include "AAA_24G_RF.h"
#include "aaa_public_KeyValue.h"

//------------tx power define-------------------
#define DEFAULT_NORMAL_TX_POWER   RF_POWER_8dBm
#define DEFAULT_PAIR_TX_POWER	  RF_POWER_8dBm
#define DEFAULT_EMI_TX_POWER      RF_POWER_8dBm

//--------------------function defein----------------------------------------

//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
#define  IDLE_TIME_COUNT_UINT 1000000

#define  CONNECT_TIME_OUT  600 //unit 1s

#define ADV_TIMER_OUT  30//unit 1S

#define BACK_LIGHT_TIME_OUT  60 //unit 10S

typedef enum
{
	POWER_ON_STATUS_AAA,
	HIGH_ADV_STATUS_AAA,
	LOW_ADV_STATUS_AAA,
	BEGIN_CONNECTED_STATUS_AAA,
	OK_CONNECTED_STATUS_AAA,
	T5S_CONNECTED_STATUS_AAA,
	DEEP_SLEEPE_STATUS_AAA,
	TERMINATE_STATUS_AAA,
	MODE_CHANGE_STATUS_AAA,
}BLE_STATUS_AAA;
typedef enum
{
	KEY_PRESS_EVENT_AAA=BIT(0),
	NEW_KEY_EVENT_AAA=BIT(1),
	SENSOR_DATA_EVENT_AAA=BIT(2),
	WHEEL_DATA_EVENT_AAA=BIT(3),
}key_EVENT_AAA;
typedef enum
{
	OS_WINDOWS_TYPE,
	OS_IOS_TYPE,
	OS_ANDROID,
}OS_TYPE_AAA;
typedef enum
{
	HAS_MOUSE_REPORT=BIT(0),
	HAS_KEYBOARD_REPORT=BIT(1),
	HAS_CONSUMER_REPORT=BIT(2),
	HAS_SYSYTEM_REPORT=BIT(3),
	HAS_JOYSTIC_REPORT=BIT(4),
}HAS_REPORT_TYPE_AAA;
typedef enum
{
	FN_PRESS_AAA=BIT(0),
	T_BIND_PRESS_AAA=BIT(1),
	MOUSE_KEY_PRESS_AAA=BIT(2),
	T_ESC_PRESS_AAA=BIT(3),
	T_Q_PRESS_AAA=BIT(4),
	T_W_PRESS_AAA=BIT(5),
	T_E_PRESS_AAA=BIT(6),
	T_R_PRESS_AAA=BIT(7),
	T_T_PRESS_AAA=BIT(8),
	T_Y_PRESS_AAA=BIT(9),
	T_U_PRESS_AAA=BIT(10),
	T_I_PRESS_AAA=BIT(11),
	T_O_PRESS_AAA=BIT(12),
	T_F1_PRESS_AAA=BIT(13),
	T_SPACE_PRESS_AAA=BIT(14),
}SPECIAL_KEY_PRESS_FLAG_AAA;

#define MAX_BTN_CNT_AAA 6
typedef struct{
	u8 cnt;
	u8 keycode[MAX_BTN_CNT_AAA]; 
	u16 special_key_press_f;
	u8 press_cnt;
}kb_data_t_aaa;
extern kb_data_t_aaa key_buf_aaa;



#define BUF_SIZE_KEYBOARD_AAA	8
#if CONSUMER_REPORT_BYTE_AAA

#define BUF_SIZE_CONSUMER_AAA	1
#else
#define BUF_SIZE_CONSUMER_AAA	4
#endif
#define BUF_SIZE_MOUSE_AAA	4


#define TX_FIFO_NUM_AAA   16
#define TX_BUF_SIZE_AAA   10


typedef struct
{
	u8 fifo[TX_FIFO_NUM_AAA][TX_BUF_SIZE_AAA];
	u8 wptr;
	u8 rptr;
	
}TX_FIFO_AAA_STRUCT;
extern TX_FIFO_AAA_STRUCT tx_fifo_aaa;

typedef struct
{
	u8 type;
	u8 len;
	u8 buf[TX_FIFO_NUM_AAA-2];
}TX_PACKET_AAA;

typedef struct
{
	u32 dongle_id;
	u8 mode;
	u8 slave_mac_addr;
	int idx;
}FLASH_DEV_INFO_AAA;
extern FLASH_DEV_INFO_AAA flash_dev_info;

typedef struct{
	u32   dev_mac; //00
	u16   group_id;//04~05
	u16  rf_vid;	//06~07
	u8  bat_type;	//008
	u8	 tx_power;	//09
	u8	 paring_tx_power;//0a
	u8   emi_tx_power;   //00b

} custom_cfg_t;
 extern custom_cfg_t   user_cfg;


extern u8 deep_flag;
extern u8 pair_flag;

extern u32 row_pins[];
extern u32 col_pins[];
extern u8 fun_mode;
extern u32 mac_id;
extern u8 ble_status_aaa;


extern u32		advertise_begin_tick;
extern int binding_master_addr_idx;
extern u8 binding_master_addr[];
extern u8 device_info_type;
extern u8 t8ms_flag;

extern u8 mouse_buf_aaa[];
extern u8 mouse_btn_in_sensor;

extern u8 has_new_report_aaa;
extern u8 combination_flag;
extern u8 kb_led_status;

extern u32 idle_tick;
extern u32  idle_count; 
extern u32 loop_cnt;
extern u8 timercnts;
extern u16 adv_timer_count;
extern u32 adv_led_interval;
u8 rcvsladata(u8 sla,u8 *s,u8 no);
u8 sendsladata(u8 sla,u8 subaddr,u8 cdata);
void driver_i2c_init(void);

#if HAS_BACK_LIGHT_AAA
extern u32 back_light_switch_flag;
extern int back_light_flash_idx;

#endif
#if SENSOR_FUN_ENABLE_AAA
extern u8 sensor_fun;
#endif
#if BATT_CHECK_FUN_ENABLE
extern u8 low_bat_count;
#endif
extern ble_sts_t  blc_att_setServerDataPendingTime_upon_ClientCmd(u8 num_10ms);

extern void ble_pair_process_aaa();
extern void clear_pair_flag();
extern void set_pair_flag();

extern void d24_start_pair();
extern void reset_idle_status();
extern void	rf_power_enable (int en);

extern u8 clear_press_key_aaa();
extern void d24_main_loop();
extern void d24_user_init();
extern void d24_check_pair_process();

extern void ble_main_loop_aaa();
extern void ble_user_init_aaa();
extern u8 ble_notify_data_proc_aaa();

extern void enter_deep_aaa();
extern void battery_check();
extern void battery_hw_init(void);

extern u8 get_data_report_aaa();
extern void add_soft_time();
extern void delet_soft_time();

extern void ui_loop();
extern u8 check_notify_fifo_empty_aaa();
extern void check_connect_interval();

extern void deep_wakeup_proc_aaa();
extern _attribute_ram_code_ int	device_send_packet (u8 * p, u32 timeout, int retry, int pairing_link);
extern int flash_info_load_aaa(u32 s_addr, u8 *d_addr,  u8 len );
extern void idle_status_poll();
extern _attribute_ram_code_  void kb_led_out_aaa(u8 status);

extern void led_hw_init();
extern void led_num_set(u8 status);
extern void led_caps_set(u8 status);
extern void led_scroll_set(u8 status);
extern void led_ble_Adv_poll();
extern void led_24g_Adv_poll();
extern void led_ConnectedStatus();
extern void led_second_sleep();
extern void led_power_on();

extern void led_bat_lvd_setup();
extern void led_new_event_blink_setup();
extern void led_24g_mode_setup();
extern void led_ble_mode_setup();
extern void back_light_poll();
extern void reg_display();
extern void rgb_led_onOff(u8 on);

extern void matrix_init_aaa();
extern u8 matrix_scan_aaa();
extern void set_special_key_flag_aaa(u8 key_code);
extern void save_key_aaa(u8 key_code);



extern void power_off_to_second_aaa();
extern void power_on_from_second_aaa();

extern void sensor_sleep();
extern void sensor_power_on();
extern u8 sensor_readData_aaa();
extern void sensor_gpio_init();


extern u8 i2c_sim_read_byte(int last);

extern void i2c_sim_start();
extern void i2c_sim_stop();
extern int i2c_sim_write_byte(u8 dat);


#endif
